/* Width test's div (TO DELETE IN PROD) */
div#mq-test {
    position: fixed;
    top: 0;
    right: 0;
    height: 50px;
    width: 50px;
    background-color: black;
    display: flex;
    justify-content: center;
    align-items: center;
    color: white;
    font-family: Cuprum, sans-serif;
    font-size: 28px;
    z-index: 9999;
}

div#mq-test::before {
    content: "G"
}

        /* Position */

        /* Dimensions */

        /* Style */

        /* For child properties */


// TEMPORAIRE //
input {
    border-style: solid;
    border-color: black;
    border-width: 1px;
    border-radius: 3px;
    padding: 2px;
}




/*
    ##################################################
    ##################### GENERAL ####################
    ##################################################
*/

html {

    body {
        /* Position */

        /* Dimensions */
        width: 100vw;
        height: 100%;

        /* Style */

        /* For child properties */
        overflow-x: hidden;


        a:focus {
            /* Position */

            /* Dimensions */

            /* Style */

            /* Animations */
            outline: none;

            /* For child properties */
        }


        hr {
            /* Position */

            /* Dimensions */
            width: 100vw;
            height: 1px;

            /* Style */
            color: #3f56ff;
            font-size: 1px;
            border-style: solid;
            border-width: 1px;

            /* For child properties */
        }


        header#principal-header {
            /* Position */

            /* Dimensions */

            /* Style */
            background-color: white;

            /* For child properties */
            display: flex;
            flex-direction: column;
            justify-content: center;


            h1 {
                /* Position */

                /* Dimensions */

                /* Style */
                text-align: center;
                font-family: "Gentium", sans-serif;
                font-size: 40px;
                color: #505050;

                /* For child properties */
            }


            nav#website-navigation-box {
                /* Position */

                /* Dimensions */

                /* Style */
                background-color: #3f56ff;

                /* For child properties */


                ul {
                    /* Position */

                    /* Dimensions */
                    padding: 0;

                    /* Style */

                    /* For child properties */
                    display: flex;
                    justify-content: center;
                    align-items: center;
                    flex-wrap: wrap;


                    li {
                        /* Position */

                        /* Dimensions */

                        /* Style */
                        list-style: none;

                        /* For child properties */
                        display: flex;
                        justify-content: center;
                        align-items: center;


                        a, input[type=submit] {
                            /* Position */

                            /* Dimensions */
                            padding: 5px;
                            margin: 5px;

                            /* Style */
                            text-decoration: none;
                            color: #3f56ff;
                            background-color: white;
                            border-style: solid;
                            border-color: #3f56ff;
                            border-width: 1px;
                            border-radius: 2px;
                            cursor: pointer;
                            font-family: "Lato", sans-serif;
                            font-size: 18px;

                            /* Animations */
                            transition-duration: 150ms;

                            /* For child properties */

                            &:hover {
                                background-color: #3f56ff;
                                color: white;
                                border-color: white;
                            }
                        }
                    }
                }
            }


            nav#administration-navigation-box {
                /* Position */

                /* Dimensions */
                padding: 5px;

                /* Style */
                color: #3f56ff;

                /* For child properties */

                a {
                    /* Position */

                    /* Dimensions */
                    padding: 7px;
                    margin: 5px;

                    /* Style */
                    background-color: #3f56ff;
                    color: white;
                    font-size: 17px;
                    font-family: Lato, sans-serif;
                    border-radius: 10px;
                    text-decoration: none;
                    text-align: center;
                    transition-duration: 200ms;

                    /* For child properties */
                    display: inline-block;

                    & :hover {
                        background-color: #464646;
                        padding: 10px;
                        /*font-size: 20px;*/
                    }
                }
            }
        }


        ul#messages-box {
            /* Position */
            margin: 0;
            padding: 0;
            position: fixed;
            bottom: 0;
            left: 0;
            right: 0;

            /* Dimensions */
            max-height: 30vh;

            /* Style */
            background-color: black;

            /* For child properties */


            li {
                /* Position */

                /* Dimensions */
                height: 4vh;

                /* Style */
                font-size: 2.5vh;
                font-family: Lato, sans-serif;
                border-top-style: solid;
                border-top-width: 1px;
                border-top-color: black;

                /* For child properties */
                display: flex;
                align-items: center;
                justify-content: center;
            }


            li.success {
                background-color: #1bee42;
            }


            li.error {
                background-color: #ff4200;
            }
        }
    }
}