@import "../../../../../admin/static/admin/css/style/base_page.scss";
@import "../background_style/0_node_handling_and_creation_general_pages.scss";
@import "../background_style/1_node_handling_and_creation_small_pages.scss";
@import "../background_style/2_node_handling_and_creation_medium_pages.scss";
@import "../background_style/3_node_handling_and_creation_large_pages.scss";
@import "../background_style/dual-listbox.scss";