WebsiteSettings_preview_fields = False

WebsiteSettings_fields_infos = {"maintenance": {"type": "checkbox",
                                                "label": "Maintenance"}
                                }